#ifndef GLAREA_H_
#define GLAREA_H_

#include <vector>

#include <QtOpenGL/QGLWidget>
#include <QtOpenGL/QGLShaderProgram>

class GLArea:public QGLWidget
{
Q_OBJECT 
public:
  GLArea (QWidget * parent = 0);

	QString getCurrentVertexShader() const;
	QString getCurrentFragmentShader() const;

public slots:
	/* controls for the gui */
	void makeTorus();
	void makeCylinder();

	void setTexturedOrWireframe(bool mode);
	void setSeamlessTextureU(bool mode);
	void setSeamlessTextureV(bool mode);
	void setMipmap(bool mode);
	void setBilinear(bool mode);
	void setTextureNumber(int textIndex);

protected:
  /// opengl initialization and drawing calls
  void initializeGL ();
  void paintGL ();
	/// mouse event callbacks
  void mousePressEvent(QMouseEvent*e);
  void mouseMoveEvent(QMouseEvent*e);
  void wheelEvent(QWheelEvent*e); 
private:
	/* a small makeshift trackball */
	float eyeDist, eyePhi, eyeTheta;
	QPointF lastMousePos;

	/*geometry*/
	std::vector<GLuint> indices; // vertex indices

	/* rendering settings */
	bool showWireframed;
	bool showSeamlessTextureU, showSeamlessTextureV;
	bool showMipmap, showBilinear;

	void setTextureOptions();

	void makeShaders();
	/* shaders */
	QGLShaderProgram
		progSeamless[2][2],  // a 2x2 set of programs (seamless in U, or in V)
		progPlain;

	/* builds fragment and vertex shaders sources */
	static QString getVertexShader(bool seamlessU, bool seamlessV, bool useTexture);
	static QString getFragmentShader(bool seamlessU, bool seamlessV, bool useTexture);
	static QString getVaryingsDeclarations(bool seamlessU, bool seamlessV, bool useTexture);

 };

#endif /*GLAREA_H_ */
